//-----------------------------------------------------------------------------
// File:     HelloMath.cpp
// Class:    None
// Parent:   None
// Children: None
// Purpose:  Tests out various mathematical functions
//-----------------------------------------------------------------------------
// The following are standard C/C++ header files.
// If a filename is enclosed inside < >  it means the header file is in the Include directory.
// If a filename is enclosed inside " "  it means the header file is in the current directory.
#include <ctype.h>      // Character Types
#include <math.h>       // Mathematical Constants
#include <stdarg.h>     // Variable Argument Lists
#include <stdio.h>      // Standard Input/Output Functions
#include <stdlib.h>     // Utility Functions
#include <string.h>     // String Operations
#include <signal.h>     // Signals (Contol-C + Unix System Calls)
#include <setjmp.h>     // Nonlocal Goto (For Control-C)
#include <time.h>       // Time and Date information
#include <assert.h>     // Verify Program Assertion
#include <errno.h>      // Error Codes (Used in Unix system())
#include <float.h>      // Floating Point Constants
#include <limits.h>     // Implementation Constants
#include <stddef.h>     // Standard Definitions
#include <exception>    // Exception handling (e.g., try, catch throw)
#include <iostream>
#include <string>
#include <cstdio>
#include <cstdlib>
using namespace std;


//-----------------------------------------------------------------------------
// Prototypes for local functions (functions not called by code in other files)
//-----------------------------------------------------------------------------
double ConvertFromRadiansToDegrees( double angleInDegrees) { return angleInDegrees * 0.017453292519943295769236907684886127134428718885417254560971914402; }
double ConvertFromDegreesToRadians( double angleInRadians) { return angleInRadians * 57.295779513082320876798154814105170332405472466564321549160243861; }
bool  WriteDoubleToFileWithExponentInSpecifiedPrecision( double x, int precision, FILE *fptr );
bool  GetStringFromFile( char inputString[], unsigned long sizeOfString, FILE* fptr);
bool  GetStringFromKeyboard( char inputString[], unsigned long sizeOfString);
bool  WriteStringToScreen ( const char outputString[]);
bool  WriteStringToFile( const char outputString[], FILE *fptr);
const char* ConvertStringToDouble( const char *s, double &returnValue, double defaultValue);
//-----------------------------------------------------------------------------
int  main( int numberOfCommandLineArguments, char *arrayOfCommandLineArguments[] )
{   
   //Prompts user to enter a angle in degrees
   WriteStringToScreen("Please enter an angle in degrees\n");
	
   //Variable to retrieve input form user
   char inputString[256];
   
   //Returns user's input as a string
   GetStringFromKeyboard(inputString,256); 	
	
   //converts string input to double, and if input was not an angle function will returns an error.
   double returnValue;
	if(ConvertStringToDouble(inputString,returnValue,45)==NULL)
	 {
		 WriteStringToScreen("A non-number was entered therefore a default value of 45 degrees will be used\n");
	 }
	
   // The following function will write the angle in degrees to the screen
   WriteStringToScreen ("\nThe angle that you entered in degrees is:");
   WriteDoubleToFileWithExponentInSpecifiedPrecision(returnValue,5,stdout);

   // The following function will write the angle in radians to the screen
   double angleInRadians=ConvertFromDegreesToRadians(returnValue);
	WriteStringToScreen ("\nThe angle that you entered in radians is:");

   // The following function will write the sine of the angle to the screen	
   WriteDoubleToFileWithExponentInSpecifiedPrecision(angleInRadians,5,stdout);
	WriteStringToScreen ("\nThe sine of the angle (in degrees) is:");
	
   // The following function will write the cosine of the angle to the screen	   
   WriteDoubleToFileWithExponentInSpecifiedPrecision(sin(angleInRadians),5,stdout);
  	WriteStringToScreen ("\nThe cosine of the angle (in degrees) is:");
   
   // The following function will write the tangent of the angle to the screen	
   WriteDoubleToFileWithExponentInSpecifiedPrecision(cos(angleInRadians),5,stdout);
	WriteStringToScreen ("\nThe tangent of the angle (in degrees) is:");
	WriteDoubleToFileWithExponentInSpecifiedPrecision(tan(angleInRadians),5,stdout);
	
   // Keep the screen displayed until the user presses the Enter key
   WriteStringToScreen( "\n\n Press  Enter  to terminate the program: " );
   int key = getchar();

   // A normal program exit returns 0 (other return values usually signal an error)
   return 0;
}


//__________________________________________________________________________________
bool GetStringFromKeyboard( char inputString[], unsigned long sizeOfString)
{ 
	//Returns the users input from the keyboard to the character inputString
   GetStringFromFile(inputString,256,stdin);
	 return 0;
}   


//_________________________________________________________________________________
bool GetStringFromFile(char inputString[], unsigned long sizeOfString, FILE* fptr)
{
	//Returns the users input from the keyboard and stores it in the inputString
   fgets (inputString, sizeOfString, fptr );
	return 0;
}

//_____________________________________________________________________________________
const char* ConvertStringToDouble( const char *s, double &returnValue, double defaultValue)
{
	// Default return value (in case the string is not a valid number)
   returnValue = defaultValue;
	
   // Check if  s  is a NULL string or  "abc"  or  "123huh"  or  "(&junk#") and if not converts to double
   if (s)
	{
      // Use the standard math function strtod to parse the number
   	char *pointerToCharacterAfterNumber = NULL;
	   double x =strtod(s,&pointerToCharacterAfterNumber);
	
      // Ensure the number was not too large (overflow), such as 1.0E+999 or -1.0E-999
      if( errno==ERANGE  &&  x!=0.0 ) return NULL;

      // Ensure the character after the number is a space or '\0', not 'a' or 'z' or ...
      char characterAfterNumber = pointerToCharacterAfterNumber ? *pointerToCharacterAfterNumber : 'z';
      if( characterAfterNumber == '\0' || isspace( characterAfterNumber ) )
      {
         returnValue = x;
         return pointerToCharacterAfterNumber;
      }
   }
   return NULL;
}


//___________________________________________________________________________________________
bool  WriteStringToScreen ( const char outputString[])
{
	//Writes the string input from outputString to file, in this case the screen.
   WriteStringToFile(outputString, stdout);
	return 0;
}


//___________________________________________________________________________________
bool  WriteStringToFile( const char outputString[], FILE *fptr)
{
	//Writes the string input from outputString to file, in this case the screen.
   fprintf(fptr,outputString);
	return 0;
}


//____________________________________________________________________________________________________________
bool  WriteDoubleToFileWithExponentInSpecifiedPrecision( double x, int precision, FILE *fptr )
{
   // Ensure the precision (number of digits in the mantissa after the decimal point) makes sense.
   // Next, calculate the field width so it includes one extra space to the right of the number.
   if( precision < 0 || precision > 17 ) precision = 15;
   int fieldWidth = precision + 8;

   // Create the format specifier and print the number
   char format[20];
   sprintf( format, " %%- %d.%dE", fieldWidth, precision );
   
   //Returns a error message if unable to write double
   return fprintf( fptr, format, x) >0;
}
